#!/usr/bin/perl -w

# -----------------------------------------------------------------------
# step2html 1.0
#
# step2html is a program which converts a STEP file into a HTML file. 
# The entities, links and linenumbers of the STEP file are highlighted 
# and HTML links are generated to jump to another line.
# If the cursor is moved over a link the target entity is displayed.
#
#  Copyright by Joern Hameister (2005)
# -----------------------------------------------------------------------



my $file = "";

# Hash with all lines of the original stepfile
my %stepfileHash;

# Hash with all lines for the output stepfile
my %outputHash;

# Hash with the reverse graph
my %revGraph;

# -----------------------------------------------------------------------
# TODO Change this algorithm
#
#
# -----------------------------------------------------------------------
sub parseLines {
	print "Parse file\n";
	return split /;/, $_[0];
}


# -----------------------------------------------------------------------
# Sort Algorithm
# All Entities with beginning '(' are put to the end of the file.
#
# -----------------------------------------------------------------------
sub sortEntityName { 
	$firstSign1 = substr($stepfileHash{$a},0,1); 
	$firstSign2 = substr($stepfileHash{$b},0,1); 
	if($firstSign1 eq "(" & $firstSign2 eq "(") { 
		return 0;
	} 
	elsif($firstSign1 eq "(") { 
		return 1;
	} 
	elsif($firstSign2 eq "(") { 
		return -1;
	} 
	else { 
		$stepfileHash{$a} cmp $stepfileHash{$b}
	}
}

# -----------------------------------------------------------------------
# Remove all carriage returns
#
#
# -----------------------------------------------------------------------
sub removeCarriageReturns {
	my $file = "";
	my $line = "";
	while(defined ($line = shift @_)) {
		# Remove newlines
	  chomp($line);
	  
		$file .= $line;
		# Remove carriage returns
		$file =~ s/\r$//g;
	}
	return $file;
}

# -----------------------------------------------------------------------
# Generate a reverse graph.
#
#
# -----------------------------------------------------------------------
sub generateReverseGraph {
	print "Generate reverse graph\n";
	my($stepfileHash,$revGraph) = @_;

	foreach $key (sort keys %stepfileHash) {
		my $stepfileline = $stepfileHash{$key};
		
		my @links;
		if(@links = $stepfileline =~ /(#\d+)/g) {
			#print "LINE: $key=$stepfileline\n";
			foreach $link (@links) {
				if (exists $revGraph{$link}) {
					# if an entry exists get it and append the new link
					#print "LINKLIST = $linkList\n";
					$revGraph{$link} = $revGraph{$link} . " " . $key;
					#print "Use existing for $key...\n";
				}
				else {
					# create a new entry and set the link
					$revGraph{$link} = $key;
					#print "Create new entry for $link = $revGraph{$link}\n";
				}
			}
		}
	}	
}

# -----------------------------------------------------------------------
# Examine all lines and put them into a hash.
#
#
# -----------------------------------------------------------------------
sub parseAllLineNumbers {
	print "Parse all linenumbers\n";
	
	my($stepfileHash, @list) = @_;
	for( $i=0; $i+1<@list; $i++) {
		my $line = $list[$i];
		my $linenumber;
		my $linecontent;
		
		# print "$line\n";
		
		# Determine linenumbers and line content
		# and put them into a hash
		if($line =~ /(#\d*)=(.*)/) {
			$linenumber = $1;
			$linecontent = $2;
			#	print "$linenumber\n";
			#	print "$linecontent\n";
			$stepfileHash{$1} = $2;
		}
		else {
			my $header;
			# Header
			if (exists $stepfileHash{"HEADER"}) {
				$header = $stepfileHash{"HEADER"};
			}
			else {
				$header = "";
			}
			$header = $header . $line . ";<BR>";
			$stepfileHash{"HEADER"} = $header;
		}
	}
	print "Finished parsing\n";
}


# -----------------------------------------------------------------------
# Write the HTML header and the JavaScript into the HTML file.
#
#
# -----------------------------------------------------------------------
sub writeHeaderAndScript(*;$@) {
	
print OUTPUT "<html><head><title>$_[1]</title>\n";

print OUTPUT "<script>
   <!-- 
      tt = null;
      document.onmousemove = updateTT;
      function updateTT(e){
        x = (document.all) ? window.event.x + document.body.scrollLeft + 90 : e.pageX;
        y = (document.all) ? window.event.y + document.body.scrollTop       : e.pageY;
        if(tt != null){
          tt.style.left = (x - 160) + \"px\";
          tt.style.top 	= (y + 20) + \"px\";
        }
      }
      function showTT(id){
        tt = document.getElementById(id);
        tt.style.display = \"block\"
      }
      function hideTT(){
        tt.style.display = \"none\";
      }
    //-->
    </script>
    <style>
    <!--
      body, td {
        font-family: \"Courier new\", Courier, mono;
        font-size: 16px;
      }
      .tooltip {
        position: absolute;
        display: none;
        background-color: #E4E4E4;
        border-width: 1px;
        border-style: solid;
        border-color: #000000;
        padding: 1px;
      }
    //-->
    </style>
    </head><body>
";
}


# -----------------------------------------------------------------------
#
# MAIN Programm
#
# -----------------------------------------------------------------------

if(@ARGV!=2) {
	print "This programm transforms a STEP file into a HTML file with links\nParameter 1: Inputfile\nParameter 2: Outputfile\nFor example: ./step2html.pl inputfile.stp outputfile.html\n";
}
else { 
	print "Read from $ARGV[0] (Inputfile)\n";
  print "Write to $ARGV[1] (Outputfile)\n";

	# load input file
	open(INPUT , "<", $ARGV[0]);
	open(OUTPUT, ">", $ARGV[1]);
	
	print "Remove Carriage Returns\n";
	$file = removeCarriageReturns(<INPUT>);
	print "Finished removing\n";
	
	
	print "Examine all STEP file lines\n";
	# Create a list with stepfile lines
	# The file is parse and separated by colons
	# This is used as input for the parseAllLineNumbers function
	parseAllLineNumbers(%stepfileHash, parseLines($file));
	
	print "Reverse link graph\n";
	# Generate a reverse graph
	generateReverseGraph(%stepfileHash, %revGraph);
	
	# get a list with sorted hash keys
	my @sortedKeys = sort sortEntityName keys %stepfileHash;

	print "Write HTML header\n";
	
	# Write HTML Header
	#<html><head><title>assmbly_with_axs</title>
	#<body>
	writeHeaderAndScript( OUTPUT , $ARGV[1]);

	# write Stepfile Header
	my $header = $stepfileHash{"HEADER"};
	print OUTPUT "$header\n";
	
	delete $stepfileHash{"HEADER"};
	
	print "Generate HTML file\n";
	foreach $sortedKey (@sortedKeys) {
	#foreach $sortedKey (sort keys %stepfileHash) {
	
	  if ($sortedKey cmp "HEADER") {
			# Get value
			$value = $stepfileHash{$sortedKey};
			$divtext = $value; 
			
			$_ = $value;
		
		  # Set color for text in colons 
		  $value =~ s/('.+')/<font color="#bbbb00">$1<\/font>/g;
	
			if( exists $revGraph{$sortedKey}) {
		  	$backrefs = $revGraph{$sortedKey};
		  	$value = $value . "<font color=\"#ff0000\"> <= </font>" . $backrefs;
		  }
		  #else {
		  #	print "NO BACKREFS for $sortedKey.\n";
		  #}
		
			# Generate links for the line ($value)
			# <a href="#61">#61<a/>
			$value =~ s/(#\d+)/<a href="$1" onMouseOver="showTT('$1')" onMouseOut="hideTT()">$1<\/a>/g; 
			#print "$_\n";
			
			# Extract link id
			if($sortedKey =~ /(#)(\d*)/) {
				$link = $2;
			}
	
			print OUTPUT "<div class=\"tooltip\" id=\"$sortedKey\">$divtext</div>\n";
		
			# <a id="61" name="61"></a> 
			print OUTPUT "<a id=\"$link\" name=\"$link\"></a> ";
			
			# Write line
			print OUTPUT "<font color=\"#ff0000\">$sortedKey</font> = <font color=\"#00aa00\">$value</font>; <BR>\n";
		}
	}
	
	# Empty lines for Scrolling
	for($i=0;$i<100;$i++) {
		print OUTPUT "<BR>";
	}
	
	# Write End
	print OUTPUT "</BODY></HTML>";
	
	print "End of Processing...\n";
}